<?php
if (!defined('ABSPATH')) {
    exit;
}

add_action('wp_ajax_vegavend_update_account', 'vegavend_update_account_handler');

/**
 * Handles the AJAX request to update the VegaVend account.
 */
function vegavend_update_account_handler() {

    // Verify nonce
    if (!check_ajax_referer('vegavend_nonce', 'vegavend_nonce', false)) {
        wp_send_json_error(array('message' => 'Nonce verification failed.'));
        return;
    }

    // Check user capabilities
    if (!current_user_can('manage_options')) {
        wp_send_json_error(array('message' => 'User does not have permission.'));
        return;
    }

    $account_data = $_POST;

    // Exclude common WordPress form fields
    unset($account_data['vegavend_nonce']);
    unset($account_data['_wpnonce']);
    unset($account_data['_wp_http_referer']);

    // Handle warranty policy separately
    if (isset($account_data['warranty_policy'])) {
        $warranty_policy = array(
            'addon_settings' => array(),
            'policy' => sanitize_textarea_field($account_data['warranty_policy']),
            'label' => sanitize_text_field($account_data['warranty_policy_label']),
            'type' => sanitize_text_field($account_data['warranty_policy_type']),
            'length' => sanitize_text_field($account_data['warranty_policy_length']),
            'length_value' => sanitize_text_field($account_data['warranty_policy_length_value']),
            'length_duration' => sanitize_text_field($account_data['warranty_policy_length_duration']),
            'reasons' => isset($account_data['warranty_policy_reasons']) && is_array($account_data['warranty_policy_reasons']) ? array_map('sanitize_text_field', $account_data['warranty_policy_reasons']) : array()
        );
        update_option('vegavend_warranty_policy', maybe_serialize($warranty_policy));
        unset($account_data['warranty_policy']);
        unset($account_data['warranty_policy_label']);
        unset($account_data['warranty_policy_type']);
        unset($account_data['warranty_policy_length']);
        unset($account_data['warranty_policy_length_value']);
        unset($account_data['warranty_policy_length_duration']);
        unset($account_data['warranty_policy_reasons']);
    }

    // Handle shipping methods separately
    if (isset($account_data['shipping_methods']) && is_array($account_data['shipping_methods'])) {
        $shipping_methods = array_map(function($method) {
            return array(
                'zone' => isset($method['zone']) ? sanitize_text_field($method['zone']) : '',
                'method' => isset($method['method']) ? sanitize_text_field($method['method']) : '',
                'cost' => isset($method['cost']) ? sanitize_text_field($method['cost']) : '',
                'free_shipping_min_amount' => isset($method['free_shipping_min_amount']) ? sanitize_text_field($method['free_shipping_min_amount']) : ''
            );
        }, $account_data['shipping_methods']);
        update_option('vegavend_shipping_methods', maybe_serialize($shipping_methods));
        unset($account_data['shipping_methods']);
    }

    // Handle social profiles separately
    if (isset($account_data['social_profiles'])) {
        $social_profiles = array_map('sanitize_text_field', $account_data['social_profiles']);
        update_option('vegavend_social_profiles', maybe_serialize($social_profiles));
        unset($account_data['social_profiles']);
    }

    // List of fields that should be prefixed with 'vegavend_'
    $prefix_fields = [
        'store_name', 'store_email', 'profile_picture', 'store_banner', 'company_name',
        'store_address_street', 'store_address_street_2', 'store_address_city', 'store_address_postcode',
        'store_address_country', 'company_id', 'support_number', 'vat_number', 'biography',
        'terms_and_conditions', 'refunds_policy', 'shipping_policy', 'store_seo_title',
        'store_seo_meta_description', 'store_seo_keywords', 'phone_number'
    ];

    foreach ($account_data as $key => $value) {
        if (is_array($value)) {
            $value = maybe_serialize($value);
        } else {
            // Use appropriate sanitization function based on the field type
            if (in_array($key, ['biography', 'terms_and_conditions', 'refunds_policy', 'shipping_policy'])) {
                $sanitized_value = sanitize_textarea_field($value);
            } else {
                $sanitized_value = sanitize_text_field($value);
            }
        }

        if (in_array($key, $prefix_fields)) {
            $option_key = 'vegavend_' . $key;
        } else {
            $option_key = $key;
        }

        // Check if the new value is different from the existing value
        $existing_value = get_option($option_key);
        if ($existing_value === $sanitized_value) {
            continue;
        }

        $update_result = update_option($option_key, $sanitized_value);

        if ($update_result === false) {
            wp_send_json_error(array('message' => 'Failed to update option: ' . $option_key));
            return;
        }
    }

    // Retrieve the unmasked store ID
    $unmasked_store_id = sanitize_text_field($account_data['unmasked_store_id']);
    if (empty($unmasked_store_id)) {
        wp_send_json_error(array('message' => 'Unmasked Store ID not found.'));
        return;
    }

    // Replace the masked store ID with the unmasked store ID
    $account_data['store_id'] = $unmasked_store_id;

    // Prepare the data to be sent to VegaVend
    $data_to_send = prepare_account_data($account_data);

    // Get the endpoint URL
    $endpoint_url = Vegavend_API_Config::get_api_url('/update-account');

    // Encode the data as JSON
    $json_data = json_encode($data_to_send);

    // Send the updated data to the VegaVend API
    $response = wp_remote_post($endpoint_url, array(
        'method' => 'POST',
        'body' => $json_data,
        'headers' => array(
            'Content-Type' => 'application/json',
            'X-API-Key' => Vegavend_API_Config::get_api_key(),
            'X-Store-ID' => $unmasked_store_id,
        ),
        'timeout' => 60, // Increase timeout duration to 60 seconds
    ));

    if (is_wp_error($response)) {
        wp_send_json_error(array('message' => 'Failed to send updated data to VegaVend.'));
        return;
    }

    $response_code = wp_remote_retrieve_response_code($response);
    $response_body = wp_remote_retrieve_body($response);

    if ($response_code !== 200) {
        wp_send_json_error(array('message' => 'Unexpected response code from VegaVend.'));
        return;
    }

    wp_send_json_success(array('message' => 'Account data updated successfully.'));
}

/**
 * Prepares the account data to be sent to VegaVend.
 *
 * @param array $account_data The account data.
 * @return array The prepared account data.
 */
function prepare_account_data($account_data) {
    $shipping_methods = maybe_unserialize(get_option('vegavend_shipping_methods'));
    $social_profiles = maybe_unserialize(get_option('vegavend_social_profiles'));
    if (!is_array($social_profiles)) {
        $social_profiles = array();
    }
    $warranty_policy = maybe_unserialize(get_option('vegavend_warranty_policy'));
    if (!is_array($warranty_policy)) {
        $warranty_policy = array(
            'addon_settings' => array(),
            'policy' => 'Add your warranty policy here.',
            'label' => 'Warranty',
            'type' => 'included_warranty',
            'length' => 'limited',
            'length_value' => '30',
            'length_duration' => 'days',
            'reasons' => array()
        );
    }

    return array(
        'store_name' => $account_data['store_name'],
        'store_email' => $account_data['store_email'],
        'profile_picture' => isset($account_data['profile_picture']) ? $account_data['profile_picture'] : '',
        'store_banner' => isset($account_data['store_banner']) ? $account_data['store_banner'] : '',
        'company_name' => $account_data['company_name'],
        'store_address_street' => $account_data['store_address_street'],
        'store_address_street_2' => $account_data['store_address_street_2'],
        'store_address_city' => $account_data['store_address_city'],
        'store_address_postcode' => $account_data['store_address_postcode'],
        'store_address_country' => $account_data['store_address_country'],
        'company_id' => $account_data['company_id_number'],
        'support_number' => $account_data['support_number'],
        'vat_number' => $account_data['vat_number'],
        'biography' => $account_data['biography'],
        'terms_and_conditions' => $account_data['terms_and_conditions'],
        'refunds_policy' => $account_data['refunds_policy'],
        'shipping_policy' => $account_data['shipping_policy'],
        'store_seo_title' => $account_data['store_seo_title'],
        'store_seo_meta_description' => $account_data['store_seo_meta_description'],
        'store_seo_keywords' => $account_data['store_seo_keywords'],
        'phone_number' => isset($account_data['phone_number']) ? $account_data['phone_number'] : '',
        'shipping_methods' => $shipping_methods,
        'social_profiles' => $social_profiles,
        'warranty_policy' => $warranty_policy,
        'store_id' => $account_data['store_id']
    );
}
