<?php
if (!defined('ABSPATH')) exit;

/**
 * Get the color for the verification status.
 *
 * @param string $status
 * @return string
 */

/**
 * Class Vegavend_Account_Page
 * Handles the account page functionality for the Vegavend plugin.
 */
class Vegavend_Account_Page {
    private static $instance = null;
    private static $routes_registered = false;
    private static $sync_in_progress = false;

    /**
     * Get the singleton instance of the class.
     *
     * @return Vegavend_Account_Page
     */
    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Constructor.
     */
    private function __construct() {
        add_action('rest_api_init', array($this, 'vegavend_register_routes'));
        add_shortcode('vegavend_account_form', array($this, 'vegavend_render_account_form'));
        add_action('wp_ajax_vegavend_get_store_id', array($this, 'vegavend_get_store_id'));
        add_action('admin_enqueue_scripts', array($this, 'vegavend_enqueue_scripts'));
        add_action('wp_ajax_vegavend_update_account', 'vegavend_update_account_handler'); // Ensure this action is correctly set up
    }

    /**
     * Register REST API routes.
     */
    public function vegavend_register_routes() {
        if (self::$routes_registered) {
            return;
        }

        register_rest_route(Vegavend_API_Config::get_base_route(), '/accounts', array(
            'methods' => 'POST',
            'callback' => array($this, 'vegavend_sync_account'),
            'permission_callback' => array($this, 'vegavend_check_api_key_permission'), 
        ));

        register_rest_route(Vegavend_API_Config::get_base_route(), '/update-account', array(
            'methods' => 'POST',
            'callback' => array($this, 'vegavend_sync_account'),
            'permission_callback' => array($this, 'vegavend_check_api_key_permission'),
        ));

        self::$routes_registered = true;
    }

    /**
     * Check API key permission for REST API requests.
     *
     * @param WP_REST_Request $request
     * @return bool|WP_Error
     */
    public function vegavend_check_api_key_permission($request) {
        $provided_api_key = $request->get_header('X-API-Key');

        if ($provided_api_key !== Vegavend_API_Config::get_api_key()) {
            return new WP_Error('invalid_api_key', esc_html__('Invalid API key.', 'vegavend-merchant-connector'), array('status' => 401));
        }
        return true;
    }

    /**
     * Handle account synchronization via REST API.
     *
     * @param WP_REST_Request $request
     * @return WP_REST_Response|WP_Error
     */
    public function vegavend_sync_account($request) {
        if (self::$sync_in_progress) {
            return new WP_Error('sync_in_progress', esc_html__('Synchronization already in progress.', 'vegavend-merchant-connector'), array('status' => 429));
        }

        self::$sync_in_progress = true;
        $store_id = $this->get_store_id_from_status_page();
        if (empty($store_id)) {
            self::$sync_in_progress = false;
            return new WP_Error('invalid_store_id', esc_html__('Store ID not found in settings.', 'vegavend-merchant-connector'), array('status' => 400));
        }

        // Acquire lock
        $lock_key = 'vegavend_sync_lock_' . $store_id;
        if (!wp_cache_add($lock_key, true, '', 30)) {
            self::$sync_in_progress = false;
            return new WP_Error('sync_in_progress', esc_html__('Synchronization already in progress.', 'vegavend-merchant-connector'), array('status' => 429));
        }

        $account_data = $request->get_json_params();
        if (empty($account_data)) {
            wp_cache_delete($lock_key); // Release lock
            self::$sync_in_progress = false;
            return new WP_Error('no_data', esc_html__('No data received.', 'vegavend-merchant-connector'), array('status' => 400));
        }

        // Handle fetch_account_data action
        if (isset($account_data['action']) && $account_data['action'] === 'fetch_account_data') {
            wp_cache_delete($lock_key); // Release lock
            self::$sync_in_progress = false;

            // Retrieve account data
            $account_data = array(
                'store_name' => get_option('vegavend_store_name'),
                'store_email' => get_option('vegavend_store_email'),
                'profile_picture' => get_option('vegavend_profile_picture'),
                'store_banner' => get_option('vegavend_store_banner'),
                'company_name' => get_option('vegavend_company_name'),
                'store_address_street' => get_option('vegavend_store_address_street'),
                'store_address_street_2' => get_option('vegavend_store_address_street_2'),
                'store_address_city' => get_option('vegavend_store_address_city'),
                'store_address_postcode' => get_option('vegavend_store_address_postcode'),
                'store_address_country' => get_option('vegavend_store_address_country'),
                'company_id' => get_option('vegavend_company_id'),
                'support_number' => get_option('vegavend_support_number'),
                'vat_number' => get_option('vegavend_vat_number'),
                'biography' => get_option('vegavend_biography'),
                'phone_number' => get_option('vegavend_phone_number'),
                'terms_and_conditions' => get_option('vegavend_terms_and_conditions'),
                'refunds_policy' => get_option('vegavend_refunds_policy'),
                'shipping_policy' => get_option('vegavend_shipping_policy'),
                'store_seo_title' => get_option('vegavend_store_seo_title'),
                'store_seo_meta_description' => get_option('vegavend_store_seo_meta_description'),
                'store_seo_keywords' => get_option('vegavend_store_seo_keywords'),
                'social_profiles' => maybe_unserialize(get_option('vegavend_social_profiles')),
                'shipping_methods' => maybe_unserialize(get_option('vegavend_shipping_methods')),
                'warranty_policy' => maybe_unserialize(get_option('vegavend_warranty_policy')),
                'payment_method' => get_option('vegavend_payment_method'),
            );

            // Ensure warranty policy is correctly retrieved
            $warranty_policy = maybe_unserialize(get_option('vegavend_warranty_policy'));
            if (!is_array($warranty_policy)) {
                $warranty_policy = array(
                    'addon_settings' => array(),
                    'policy' => 'Add your warranty policy here.',
                    'label' => 'Warranty',
                    'type' => 'included_warranty',
                    'length' => 'limited',
                    'length_value' => '30',
                    'length_duration' => 'days',
                    'reasons' => array()
                );
            }
            $account_data['warranty_policy'] = $warranty_policy;

            // Add verification status to account data without saving to database
            $account_data['verification_status'] = array(
                'identification' => get_option('vegavend_identification_verified'),
                'address' => get_option('vegavend_address_verified'),
                'company_registration' => get_option('vegavend_company_id_verified'),
                'vat_number' => get_option('vegavend_vat_number_verified')
            );

            return new WP_REST_Response(array(
                'status' => 'success',
                'data' => $account_data
            ), 200);
        }

        // Handle profile picture upload
        if (isset($_FILES['profile_picture_upload']) && !empty($_FILES['profile_picture_upload']['name'])) {
            $upload = wp_handle_upload($_FILES['profile_picture_upload'], array('test_form' => false));
            if (isset($upload['url'])) {
                $account_data['profile_picture'] = $upload['url'];
            } else {
                wp_cache_delete($lock_key); // Release lock
                self::$sync_in_progress = false;
                return new WP_Error('upload_error', esc_html__('Failed to upload profile picture.', 'vegavend-merchant-connector'), array('status' => 400));
            }
        } else {
            $account_data['profile_picture'] = get_option('vegavend_profile_picture');
        }

        // Handle store banner upload
        if (isset($_FILES['store_banner_upload']) && !empty($_FILES['store_banner_upload']['name'])) {
            $upload = wp_handle_upload($_FILES['store_banner_upload'], array('test_form' => false));
            if (isset($upload['url'])) {
                $account_data['store_banner'] = $upload['url'];
            } else {
                wp_cache_delete($lock_key); // Release lock
                self::$sync_in_progress = false;
                return new WP_Error('upload_error', esc_html__('Failed to upload store banner.', 'vegavend-merchant-connector'), array('status' => 400));
            }
        } else {
            $account_data['store_banner'] = get_option('vegavend_store_banner');
        }

        // Update store-specific data with the received account data
        foreach ($account_data as $key => $value) {
            if (is_array($value)) {
                $value = maybe_serialize($value); // Serialize arrays before saving
            } elseif (in_array($key, ['biography', 'terms_and_conditions', 'refunds_policy', 'shipping_policy', 'warranty_policy'])) {
                $value = wp_kses_post($value); // Allow formatted data for specific fields
            } else {
                $value = sanitize_text_field($value);
            }
            update_option('vegavend_' . $key, $value);
        }

        // Sync with Vegavend
        $this->vegavend_sync_with_vegavend($store_id, $account_data);

        // Send store ID to Woo account
        $this->sync_with_woo_account($store_id, $account_data);

        // Clear transient cache
        delete_transient('vegavend_account_data_' . $store_id);

        // Release lock
        wp_cache_delete($lock_key);
        self::$sync_in_progress = false;

        return new WP_REST_Response(array(
            'status' => 'success',
            'data' => $account_data
        ), 200);
    }

    /**
     * Sync account data with WooCommerce.
     *
     * @param string $store_id
     * @param array $account_data
     * @param int $retryCount
     * @return WP_Error|void
     */
    private function sync_with_woo_account($store_id, $account_data, $retryCount = 0) {
        $response = wp_remote_post(Vegavend_API_Config::get_account_url(), array(
            'method' => 'POST',
            'headers' => array(
                'Content-Type' => 'application/json',
                'X-API-Key' => Vegavend_API_Config::get_api_key(),
                'X-Store-ID' => $store_id,
            ),
            'body' => json_encode($account_data),
            'timeout' => 30, // Increase timeout duration further
        ));

        if (is_wp_error($response)) {
            return new WP_Error('sync_failed', esc_html__('Oops! Something went wrong, please try again.', 'vegavend-merchant-connector'), array('status' => 500));
        }

        $response_code = wp_remote_retrieve_response_code($response);
        if ($response_code === 429) {
            return new WP_Error('rate_limited', esc_html__('Rate limit exceeded, please try again later.', 'vegavend-merchant-connector'), array('status' => 429));
        } elseif ($response_code !== 200) {
            return new WP_Error('sync_failed', esc_html__('Oops! Something went wrong, please try again.', 'vegavend-merchant-connector'), array('status' => 500));
        }
    }

    /**
     * Sync account data with Vegavend.
     *
     * @param string $store_id
     * @param array $account_data
     * @param int $retryCount
     * @return WP_Error|void
     */
    private function vegavend_sync_with_vegavend($store_id, $account_data, $retryCount = 0) {
        $vegavend_fields = array(
            'profile_picture' => 'vegavend_profile_picture',
            'store_banner' => 'vegavend_store_banner',
            'store_address_street' => 'vegavend_store_address_street',
            'store_address_street_2' => 'vegavend_store_address_street_2',
            'store_address_city' => 'vegavend_store_address_city',
            'store_address_postcode' => 'vegavend_store_address_postcode',
            'store_address_country' => 'vegavend_store_address_country',
            'company_name' => 'vegavend_company_name',
            'company_id' => 'vegavend_company_id',
            'support_number' => 'vegavend_support_number',
            'vat_number' => 'vegavend_vat_number',
            'biography' => 'vegavend_biography',
            'terms_and_conditions' => 'vegavend_terms_and_conditions',
            'shipping_methods' => isset($account_data['shipping_methods']) ? maybe_serialize($account_data['shipping_methods']) : '', // Serialize arrays
            'shipping_policy' => 'vegavend_shipping_policy',
            'refunds_policy' => 'vegavend_refunds_policy',
            'warranty_policy' => 'warranty_policy', // Add missing comma
            'social_profiles' => isset($account_data['social_profiles']) ? maybe_serialize($account_data['social_profiles']) : '', // Serialize array
            'store_seo' => 'vegavend_store_seo',
            'store_seo_title' => 'vegavend_store_seo_title',
            'store_seo_meta_description' => 'vegavend_store_seo_meta_description',
            'store_seo_keywords' => 'vegavend_store_seo_keywords',
            'phone_number' => 'vegavend_phone_number',
            'store_name' => 'vegavend_store_name',
            'store_email' => 'vegavend_store_email',
        );

        foreach ($vegavend_fields as $key => $option_key) {
            if (isset($account_data[$key])) {
                if ($key === 'social_profiles' || $key === 'shipping_methods' || $key === 'warranty_policy') {
                    update_option($option_key, maybe_serialize($account_data[$key]));
                } else {
                    update_option($option_key, sanitize_text_field($account_data[$key]));
                }
            }
        }

        // Check if Stripe account is linked
        if (isset($account_data['payment_method'])) {
            update_option('vegavend_payment_method', sanitize_text_field($account_data['payment_method']));
            if ($account_data['payment_method'] === 'Stripe') {
                update_option('vegavend_stripe_connected', 'yes');
            } else {
                update_option('vegavend_stripe_connected', 'no');
            }
        }

        $response = wp_remote_post(Vegavend_API_Config::get_account_url(), array(
            'method' => 'POST',
            'headers' => array(
                'Content-Type' => 'application/json',
                'X-API-Key' => Vegavend_API_Config::get_api_key(),
                'X-Store-ID' => $store_id,
            ),
            'body' => json_encode($account_data),
            'timeout' => 30, // Increase timeout duration further
        ));

        if (is_wp_error($response)) {
            return new WP_Error('sync_failed', esc_html__('Oops! Something went wrong, please try again.', 'vegavend-merchant-connector'), array('status' => 500));
        }

        $response_code = wp_remote_retrieve_response_code($response);
        if ($response_code === 429) {
            return new WP_Error('rate_limited', esc_html__('Rate limit exceeded, please try again later.', 'vegavend-merchant-connector'), array('status' => 429));
        } elseif ($response_code !== 200) {
            return new WP_Error('sync_failed', esc_html__('Oops! Something went wrong, please try again.', 'vegavend-merchant-connector'), array('status' => 500));
        }
    }

    /**
     * Get shipping methods for the store.
     *
     * @param string $store_id
     * @return array
     */
    private function vegavend_get_shipping_methods($store_id) {
        $shipping_methods = array();
        $zones = WC_Shipping_Zones::get_zones();

        foreach ($zones as $zone) {
            $zone_obj = new WC_Shipping_Zone($zone['zone_id']);
            $methods = $zone_obj->get_shipping_methods();

            foreach ($methods as $method) {
                $free_shipping_min_amount = null;
                if ($method->id === 'free_shipping' && isset($method->min_amount)) {
                    $free_shipping_min_amount = $method->min_amount;
                }

                $shipping_methods[] = array(
                    'zone' => $zone['zone_name'],
                    'method_id' => $method->id,
                    'method_title' => $method->title,
                    'cost' => isset($method->cost) ? $method->cost : '',
                    'free_shipping_min_amount' => $free_shipping_min_amount,
                );
            }
        }

        return $shipping_methods;
    }

    public function render_account_page() {
        echo do_shortcode('[vegavend_account_form]');
    }

    public function vegavend_get_store_id() {
        check_ajax_referer('vegavend_nonce', 'vegavend_nonce');
        $store_id = get_option('vegavend_store_id');
        if ($store_id) {
            wp_send_json_success(array('store_id' => $store_id));
        } else {
            wp_send_json_error();
        }
    }

    private function get_store_id_from_status_page() {
        // Fetch the store ID from the status page
        $store_id = get_option('vegavend_store_id');
        return $store_id;
    }

    public function vegavend_enqueue_scripts() {
        wp_enqueue_script('jquery');
        wp_enqueue_script('wp-tinymce'); // Ensure TinyMCE is enqueued
        wp_enqueue_media(); // Enqueue WordPress media library
        wp_enqueue_script('vegavend-account-script', plugins_url('/assets/js/vegavend-account.js', dirname(__FILE__)), array('jquery'), time(), true);
        wp_enqueue_style('vegavend-account-css', plugins_url('/assets/css/vegavend-account.css', dirname(__FILE__)), array(), time(), 'all');
        wp_localize_script('vegavend-account-script', 'vegavendData', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'restUrl' => Vegavend_API_Config::BASE_URL . '/wp-json/', // Use BASE_URL
            'apiKey' => Vegavend_API_Config::get_api_key(),
            'storeId' => get_option('vegavend_store_id'), // Ensure store ID is included
            'policyDisclaimerUrl' => site_url('/policy-disclaimer'),
            'acceptDisclaimerText' => __('I Understand', 'vegavend-merchant-connector'),
            'termsTitle' => __('Terms and Conditions', 'vegavend-merchant-connector'),
            'overviewText' => __('OVERVIEW', 'vegavend-merchant-connector'),
            'serviceText' => __('This store is operated by [VENDOR STORE NAME]. Throughout the site, the terms “we”, “us” and “our” refer to [VENDOR STORE NAME]. [VENDOR STORE NAME] offers this store, including all information, tools and services available from this site to you, the user, conditioned upon your acceptance of all terms, conditions, policies and notices stated here.', 'vegavend-merchant-connector'),
            'readTermsText' => __('By visiting our site and/ or purchasing something from us, you engage in our “Service” and agree to be bound by the following terms and conditions (“Terms of Service”, “Terms”), including those additional terms and conditions and policies referenced herein and/or available by hyperlink. These Terms of Service apply to all users of the site, including without limitation users who are browsers, vendors, customers, merchants, and/ or contributors of content.', 'vegavend-merchant-connector'),
            'newFeaturesText' => __('Any new features or tools which are added to the current store shall also be subject to the Terms of Service. You can review the most current version of the Terms of Service at any time on this page. We reserve the right to update, change or replace any part of these Terms of Service by posting updates and/or changes to our store. It is your responsibility to check this page periodically for changes. Your continued use of or access to the store following the posting of any changes constitutes acceptance of those changes.', 'vegavend-merchant-connector'),
            'hostedText' => __('Our store is hosted on VegaVend. They provide us with the online e-commerce platform that allows us to sell our products and Services to you.', 'vegavend-merchant-connector'),
            'woocommerceStoreAddress' => get_option('woocommerce_store_address'),
            'woocommerceStoreAddress2' => get_option('woocommerce_store_address_2'),
            'woocommerceStoreCity' => get_option('woocommerce_store_city'),
            'woocommerceStorePostcode' => get_option('woocommerce_store_postcode'),
            'woocommerceStoreCountry' => WC()->countries->get_base_country()
        ));
    }

    public function vegavend_render_account_form() {
        // Log to check if cached data is being used
        $store_id = get_option('vegavend_store_id');
        $cached_data = get_transient('vegavend_account_data_' . $store_id);

        ob_start();
        include plugin_dir_path(__FILE__) . 'templates/vegavend-account-form.php';
        return ob_get_clean();
    }
}

Vegavend_Account_Page::get_instance();
?>