<?php
if (!defined('ABSPATH')) exit;

require_once plugin_dir_path(__FILE__) . 'vegavend_api_config.php';

class Vegavend_Orders_Import {

    public $version = '1.0.0';
    private static $route_registered = false;

    public function __construct() {
        add_action('rest_api_init', function() {
            register_rest_route('vegavend-store-api/v1', '/orders', array(
                'methods' => 'POST',
                'callback' => array($this, 'handle_api_request'),
                'permission_callback' => array($this, 'check_api_key_permission'),
            ));
        });
    }

    public function check_api_key_permission($request) {
        // Check API key permission
        $provided_api_key = $request->get_header('X-API-Key');
        if ($provided_api_key !== Vegavend_API_Config::get_api_key()) {
            return new WP_Error('invalid_api_key', __('Invalid API key.', 'vegavend-merchant-connector'), array('status' => 401));
        }
        return true;
    }

    public function include_files() {
        require_once(ABSPATH . 'wp-admin/includes/media.php');
        require_once(ABSPATH . 'wp-admin/includes/file.php');
        require_once(ABSPATH . 'wp-admin/includes/image.php');
    }

    // Handle API request for creating or updating orders
    public function handle_api_request($request) {
        $api_key = sanitize_text_field($request->get_header('X-Api-Key'));
        $data = $request->get_json_params();

        // Sanitize input data
        $data = $this->sanitize_recursive($data);

        update_option('vegavend_sync_order_debug_recived_api', $api_key);

        if (empty($api_key)) {
            return new WP_Error('missing_api_key', __('Missing API key', 'vegavend-merchant-connector'), array('status' => 400));
        }

        if (empty($data['order'])) {
            return new WP_Error('missing_data', __('Missing order data', 'vegavend-merchant-connector'), array('status' => 400));
        }

        if ($api_key == Vegavend_API_Config::get_api_key()) {
            update_option('vegavend_import_order_order_woo_main', $data);
            $response = $this->process_order_data($data);
            update_option('vegavend_import_order_order__woo_the_end', $response);
            return rest_ensure_response($response);
        } else {
            return new WP_Error('invalid_api_key', __('Invalid API key', 'vegavend-merchant-connector'), array('status' => 403));
        }
    }

    private function sanitize_recursive($data) {
        if (is_array($data)) {
            foreach ($data as $key => $value) {
                $data[$key] = $this->sanitize_recursive($value);
            }
        } else {
            $data = sanitize_text_field($data);
        }
        return $data;
    }

    public function handle_order_request($request) {
        return $this->handle_api_request($request);
    }

    // Get the admin user ID
    public function get_admin_user_id() {
        $admins = get_users(array('role' => 'administrator'));
        return !empty($admins) ? $admins[0]->ID : false;
    }

    // Process the order data received from VegaVend
    public function process_order_data($woocommerce_orders) {
        
        if (!isset($woocommerce_orders['store_id']) || empty($woocommerce_orders['store_id']) || $woocommerce_orders['store_id'] == '00') {
            return ['woocommerce store id not found'];
        }

        $store_id = sanitize_text_field($woocommerce_orders['store_id']);
        $key = get_option('vegavend_store_id'); // Changed to use the correct option key
        if ($store_id != $key) {
            return ["Store id not matched"];
        }

        if (empty($woocommerce_orders['order'])) {
            return ["Orders Not found"];
        }

        $vegavend_order_number = sanitize_text_field($woocommerce_orders['order']['vegavend_order_number']);

        $order_origin = 'VegaVend Marketplace'; // Ensure order origin is set to VegaVend Marketplace

        // Check if the order already exists in WooCommerce
        $existing_order_id = $this->get_order_id_by_vegavend_order_number($vegavend_order_number);
        if ($existing_order_id) {
            return ['order_id' => $vegavend_order_number, 'response' => 'Order already added'];
        }

        $user_id = $this->get_admin_user_id();
        $response = $this->create_woocommerce_orders_api($woocommerce_orders, $user_id, $vegavend_order_number);
        return ['order_id' => $vegavend_order_number, 'response' => $response];
    }

    // Get WooCommerce order ID by VegaVend order number
    private function get_order_id_by_vegavend_order_number($vegavend_order_number) {
        $cache_key = 'vegavend_order_id_by_vegavend_order_number_' . md5($vegavend_order_number);
        $order_id = get_transient($cache_key);

        if ($order_id === false) {
            $query = new WC_Order_Query(array(
                'limit' => 1,
                'meta_key' => '_vegavend_order_number',
                'meta_value' => sanitize_text_field($vegavend_order_number),
                'return' => 'ids',
            ));
            $orders = $query->get_orders();
            $order_id = !empty($orders) ? $orders[0] : false;

            set_transient($cache_key, $order_id, HOUR_IN_SECONDS);
        }

        return $order_id;
    }

    // Format phone number to remove non-numeric characters
    public function formatPhoneNumber($phoneNumber) {
        return preg_replace('/\D/', '', $phoneNumber);
    }

    // Get product ID by original ID
    private function get_product_id_by_original_id($original_id) {
        $cache_key = 'vegavend_product_id_by_original_id_' . md5($original_id);
        $product_id = wp_cache_get($cache_key, 'custom_cache_group');

        if ($product_id === false) {
            $product_id = get_transient($cache_key);

            if ($product_id === false) {
                $product_id = get_posts(array(
                    'post_type' => 'product',
                    'meta_query' => array(
                        array(
                            'key' => '_original_product_id',
                            'value' => sanitize_text_field($original_id),
                            'compare' => '='
                        )
                    ),
                    'fields' => 'ids',
                    'posts_per_page' => 1
                ));

                if (!empty($product_id)) {
                    $product_id = $product_id[0];
                    set_transient($cache_key, $product_id, HOUR_IN_SECONDS);
                } else {
                    $product_id = false;
                }
            }

            wp_cache_set($cache_key, $product_id, 'custom_cache_group', HOUR_IN_SECONDS);
        }

        return $product_id;
    }

    // Get product ID by SKU
    private function get_product_id_by_sku($sku) {
        $cache_key = 'vegavend_product_id_by_sku_' . md5($sku);
        $product_id = wp_cache_get($cache_key, 'custom_cache_group');

        if ($product_id === false) {
            $product_id = wc_get_product_id_by_sku(sanitize_text_field($sku));

            if ($product_id) {
                wp_cache_set($cache_key, $product_id, 'custom_cache_group', HOUR_IN_SECONDS);
            } else {
                $product_id = false;
            }
        }

        return $product_id;
    }

    // Get product ID by GTIN
    private function get_product_id_by_gtin($gtin) {
        $cache_key = 'vegavend_product_id_by_gtin_' . md5($gtin);
        $product_id = wp_cache_get($cache_key, 'custom_cache_group');

        if ($product_id === false) {
            $product_id = get_transient($cache_key);

            if ($product_id === false) {
                $product_id = get_posts(array(
                    'post_type' => 'product',
                    'meta_query' => array(
                        array(
                            'key' => '_gtin',
                            'value' => sanitize_text_field($gtin),
                            'compare' => '='
                        )
                    ),
                    'fields' => 'ids',
                    'posts_per_page' => 1
                ));

                if (!empty($product_id)) {
                    $product_id = $product_id[0];
                    set_transient($cache_key, $product_id, HOUR_IN_SECONDS);
                } else {
                    $product_id = false;
                }
            }

            wp_cache_set($cache_key, $product_id, 'custom_cache_group', HOUR_IN_SECONDS);
        }

        return $product_id;
    }

    // Get product ID by MPN
    private function get_product_id_by_mpn($mpn) {
        $cache_key = 'vegavend_product_id_by_mpn_' . md5($mpn);
        $product_id = wp_cache_get($cache_key, 'custom_cache_group');

        if ($product_id === false) {
            $product_id = get_transient($cache_key);

            if ($product_id === false) {
                $product_id = get_posts(array(
                    'post_type' => 'product',
                    'meta_query' => array(
                        array(
                            'key' => '_mpn',
                            'value' => sanitize_text_field($mpn),
                            'compare' => '='
                        )
                    ),
                    'fields' => 'ids',
                    'posts_per_page' => 1
                ));

                if (!empty($product_id)) {
                    $product_id = $product_id[0];
                    set_transient($cache_key, $product_id, HOUR_IN_SECONDS);
                } else {
                    $product_id = false;
                }
            }

            wp_cache_set($cache_key, $product_id, 'custom_cache_group', HOUR_IN_SECONDS);
        }

        return $product_id;
    }

    // Create WooCommerce order from VegaVend order data
    public function create_woocommerce_orders_api($woocommerce_orders, $user_id, $vegavend_order_number) {
        $current_user = get_userdata($user_id);
        $orderData = $woocommerce_orders['order'];
        $order = wc_create_order(array('customer_id' => $user_id));
        $any_found = false;
        $total_tax = (float) ($orderData['total_tax'] ?? 0); // Ensure total_tax is defined

        foreach ($orderData['line_items'] as $item) {
            // Prioritize original product ID, then fall back to other identifiers
            $product_id = $this->get_product_id_by_original_id($item['original_product_id'] ?? '') 
                        ?: $this->get_product_id_by_sku($item['sku'] ?? '') 
                        ?: $this->get_product_id_by_gtin($item['gtin'] ?? '') 
                        ?: $this->get_product_id_by_mpn($item['mpn'] ?? '');

            if ($product_id) {
                $product = wc_get_product($product_id);

                // Check if the product is set up for tax
                $product_tax_status = $product->get_tax_status();

                // If the product is not set up for tax, add total tax to the item cost
                if ($product_tax_status !== 'taxable') {
                    $item['price'] += $total_tax / count($orderData['line_items']); // Distribute total tax across items
                    $item['total'] += $total_tax / count($orderData['line_items']); // Add tax to total cost
                }

                // Retrieve product addons from the order item data
                $product_addons = array_filter((array) $item['product_addons'] ?? array());

                // Add meta values for keys that start with a specific prefix to product addons
                if (isset($item['meta_data']) && is_array($item['meta_data'])) {
                    foreach ($item['meta_data'] as $meta) {
                        if ($meta->key === '_pao_ids') {
                            $product_addons[] = $meta->value;
                        }
                    }
                }

                // Ensure product addons are sanitized
                $product_addons = $this->sanitize_recursive($product_addons);

                $order_item = new WC_Order_Item_Product();
                $order_item->set_name($product->get_name());
                $order_item->set_product($product);
                $order_item->set_product_id($product_id);
                $order_item->set_quantity($item['quantity']);
                $order_item->set_subtotal(isset($item['price']) ? $item['price'] : 0);
                $order_item->set_total(isset($item['price']) ? $item['price'] : 0);
                $order_item->set_tax_class($orderData['tax_class'] ?? ''); // Use tax class from order data
                $order_item->set_subtotal_tax(isset($item['total_tax']) ? $item['total_tax'] : $total_tax / count($orderData['line_items']));

                // Set the tax data for the line item
                $tax_data = array(
                    'total' => array(
                        1 => isset($item['total_tax']) ? $item['total_tax'] : $total_tax / count($orderData['line_items'])
                    ),
                    'subtotal' => array(
                        1 => isset($item['total_tax']) ? $item['total_tax'] : $total_tax / count($orderData['line_items'])
                    )
                );
                $order_item->set_taxes($tax_data);

                $order_item->set_variation_id(isset($item['vegavend_variation_id']) ? $item['vegavend_variation_id'] : 0);
                $order_item->set_variation($item['variation_attributes'] ?? array());

                // Add product addons to the configuration data
                foreach ($product_addons as $addon) {
                    if (is_array($addon)) {
                        foreach ($addon as $addon_detail) {
                            $order_item->add_meta_data($addon_detail['key'], $addon_detail['value']);
                        }
                    }
                }

                $order->add_item($order_item);
                $any_found = true;
            }

            // Ensure the id is set to the original_product_id
            $item['id'] = $item['original_product_id'];
        }

        if (!$any_found) {
            return 'no product found';
        }

        // Add a custom line item for the VegaVend order number
        $order_item = new WC_Order_Item();
        $order_item->set_name('VegaVend Order# ' . $vegavend_order_number);
        $order->add_item($order_item);
        $order->save();
        
        $formattedPhoneNumber = $this->formatPhoneNumber($orderData['billing_address']['phone']);
        $billing_address = array(
            'first_name' => $orderData['billing_address']['first_name'] ?? '',
            'last_name' => $orderData['billing_address']['last_name'] ?? '',
            'company' => $orderData['billing_address']['company'] ?? '',
            'address_1' => $orderData['billing_address']['address1'] ?? '',
            'address_2' => $orderData['billing_address']['address2'] ?? '',
            'city' => $orderData['billing_address']['city'] ?? '',
            'state' => $orderData['billing_address']['province'] ?? '',
            'postcode' => $orderData['billing_address']['zip'] ?? '',
            'country' => $orderData['billing_address']['country'] ?? '',
            'email' => $orderData['customer']['email'] ?? '',
            'phone' => $formattedPhoneNumber,
        );

        $shipping_address = array(
            'first_name' => $orderData['shipping_address']['first_name'] ?? '',
            'last_name' => $orderData['shipping_address']['last_name'] ?? '',
            'company' => $orderData['shipping_address']['company'] ?? '',
            'address_1' => $orderData['shipping_address']['address1'] ?? '',
            'address_2' => $orderData['shipping_address']['address2'] ?? '',
            'city' => $orderData['shipping_address']['city'] ?? '',
            'state' => $orderData['shipping_address']['province'] ?? '',
            'postcode' => $orderData['shipping_address']['zip'] ?? '',
            'country' => $orderData['shipping_address']['country_code'] ?? '',
        );

        // Fetch the payment method, tax settings, and shipping data from the order data
        $payment_method = $this->get_human_readable_payment_method($orderData['payment_method']);
        $tax_class = $orderData['tax_class'] ?? ''; // Ensure tax_class is defined
        $tax_status = $orderData['tax_status'] ?? ''; // Ensure tax_status is defined
        $shipping_method = $orderData['shipping_method'];
        $shipping_total = (float) $orderData['shipping_total'];
        $shipping_tax = (float) $orderData['shipping_tax'];
        $total_tax = (float) $orderData['total_tax'];
        $selling_price = (float) $orderData['total_price']; // Ensure selling_price is defined
        $store_id = $orderData['store_id'] ?? ''; // Ensure store_id is defined

        // Explicitly set each _wc_order_attribution_* meta value
        //$order->update_meta_data('_wc_order_attribution_source', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_source'));
        //$order->update_meta_data('_wc_order_attribution_medium', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_medium'));
        $order->update_meta_data('_wc_order_attribution_campaign', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_campaign'));
        $order->update_meta_data('_wc_order_attribution_term', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_term'));
        $order->update_meta_data('_wc_order_attribution_content', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_content'));
        $order->update_meta_data('_wc_order_attribution_source_type', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_source_type'));
        $order->update_meta_data('_wc_order_attribution_session_pages', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_session_pages'));
        $order->update_meta_data('_wc_order_attribution_session_count', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_session_count'));
        $order->update_meta_data('_wc_order_attribution_device_type', $this->get_meta_value($woocommerce_orders['order']['order_meta'], '_wc_order_attribution_device_type'));

        // Set the order origin and source if not already set
        $order->update_meta_data('_wc_order_attribution_utm_source', 'VegaVend Marketplace');
        $order->update_meta_data('_wc_order_attribution_utm_medium', 'Web');

        // Save the meta data after setting it
        $order->save_meta_data();

        // Set the order data
        $order->set_address($shipping_address, 'shipping');
        $order->set_address($billing_address, 'billing');
        $order->set_payment_method($payment_method); // Set payment method from order data
        $order->set_shipping_total($shipping_total);
        $shipping_item = new WC_Order_Item_Shipping();
        $shipping_item->set_method_title($shipping_method);
        $shipping_item->set_total($shipping_total);
        $order->add_item($shipping_item);
        $order->set_shipping_tax($shipping_tax);
        $order->set_cart_tax($total_tax - $shipping_tax); // Set cart tax
        $order->set_total($orderData['total_price']);
        $order->add_order_note($orderData['note']);
        $order->calculate_totals();
        $total_amount = $order->get_total();

        // Set tax class for each item
        foreach ($order->get_items() as $item_id => $item) {
            $item->set_tax_class($tax_class);
        }

        if ($total_amount < $selling_price) {
            $other_charges = $selling_price - $total_amount;
            if ($other_charges > 0) {
                $fee = new WC_Order_Item_Fee();
                $fee->set_name('Other Charges');
                $fee->set_amount($other_charges);
                $fee->set_total($other_charges);
                $order->add_item($fee);
                $order->calculate_totals();
            }
       }

        // Disable email notifications
        add_filter('woocommerce_email_enabled_customer_processing_order', '__return_false');
        add_filter('woocommerce_email_enabled_customer_completed_order', '__return_false');
        add_filter('woocommerce_email_enabled_customer_invoice', '__return_false');
        add_filter('woocommerce_email_enabled_customer_note', '__return_false');

        $order->update_status('processing');
        $order->save();

        // Add a note to the order indicating it was created for a specific VegaVend order number
        $order->add_order_note('Order created for VegaVend Order# ' . $vegavend_order_number);

        // Send updated order details back to VegaVend
        $order_update = new Vegavend_Order_Update();
        $order_update->send_updated_order_to_vegavend($order);

        // Store the VegaVend order number and customer ID in the order meta
        $order->update_meta_data('_vegavend_order_number', $vegavend_order_number);
        $order->update_meta_data('_vegavend_customer_id', $orderData['customer']['vegavend_customer_id']);
        $order->update_meta_data('_vegavend_store_id', $store_id);
        $order->save_meta_data();

        // Re-enable email notifications
        remove_filter('woocommerce_email_enabled_customer_processing_order', '__return_false');
        remove_filter('woocommerce_email_enabled_customer_completed_order', '__return_false');
        remove_filter('woocommerce_email_enabled_customer_invoice', '__return_false');
        remove_filter('woocommerce_email_enabled_customer_note', '__return_false');

        return 'Order Is Created Successfully';
    }

    // Function to convert payment method to human-readable format
    private function get_human_readable_payment_method($payment_method) {
        $payment_method = str_replace('dokan-', '', $payment_method);
        $payment_method = str_replace('-', ' ', $payment_method);
        return ucwords($payment_method);
    }

    // Helper function to get meta value by key
    private function get_meta_value($meta_data, $key) {
        foreach ($meta_data as $meta) {
            if ($meta['key'] === $key) {
                return $meta['value'];
            }
        }
        return '';
    }

}

new Vegavend_Orders_Import();