<?php
if (!defined('ABSPATH')) exit;

require_once plugin_dir_path(__FILE__) . 'vegavend_api_config.php';

class Vegavend_Order_Update {
    public function __construct() {
        add_action('rest_api_init', array($this, 'register_order_api_route'));
        add_action('woocommerce_order_status_changed', array($this, 'on_order_update'), 10, 1);
        // Add filter to conditionally disable email notifications for VegaVend orders
        add_filter('woocommerce_email_enabled', array($this, 'disable_emails_for_vegavend_orders'), 10, 2);
    }

    public function register_order_api_route() {
        register_rest_route('vegavend-store-api/v1', '/orders/(?P<id>\d+)', array(
            'methods'  => 'PUT',
            'callback' => array($this, 'handle_order_update_request'),
            'permission_callback' => array($this, 'check_api_key_permission'),
        ));
    }

    public function check_api_key_permission($request) {
        // Check API key permission
        $provided_api_key = $request->get_header('X-API-Key');
        if ($provided_api_key !== Vegavend_API_Config::get_api_key()) {
            return new WP_Error('invalid_api_key', __('Invalid API key.', 'vegavend-merchant-connector'), array('status' => 401));
        }
        return true;
    }

    public function handle_order_update_request($request) {
        $vegavend_order_number = sanitize_text_field($request['id']);
        $data = $request->get_json_params();

        // Sanitize input data recursively
        $data = $this->sanitize_recursive($data);

        // Disable email notifications
        add_filter('woocommerce_email_enabled_customer_processing_order', '__return_false');
        add_filter('woocommerce_email_enabled_customer_completed_order', '__return_false');
        add_filter('woocommerce_email_enabled_customer_invoice', '__return_false');
        add_filter('woocommerce_email_enabled_customer_note', '__return_false');

        // Search for the WooCommerce order using the VegaVend order number
        $cache_key = 'order_id_by_vegavend_order_number_' . md5($vegavend_order_number);
        $order_id = get_transient($cache_key);

        if ($order_id === false) {
            $order_query = new WC_Order_Query(array(
                'meta_key' => '_vegavend_order_number',
                'meta_value' => $vegavend_order_number,
                'limit' => 1,
            ));
            $orders = $order_query->get_orders();
            $order_id = !empty($orders) ? $orders[0]->get_id() : false;

            set_transient($cache_key, $order_id, HOUR_IN_SECONDS);
        }

        if (!$order_id) {
            return new WP_Error('order_not_found', __('Order not found', 'vegavend-merchant-connector'), array('status' => 404));
        }

        $order = wc_get_order($order_id);

        // Update order details based on the data received
        $order->set_address($data['billing_address'], 'billing');
        $order->set_address($data['shipping_address'], 'shipping');
        $order->set_total($data['total_price']);
        $order->set_status($data['status']); // Update order status
        $order->add_order_note(__('Order updated from VegaVend', 'vegavend-merchant-connector'));

        // Set order origin meta data
        $order->update_meta_data('_order_origin', 'VegaVend Marketplace');
        if (isset($data['order_id'])) {
            $order->update_meta_data('_vendor_order_number', $data['order_id']); // Save vendor's WooCommerce order number
        }

        // Add additional notes if provided
        if (!empty($data['notes'])) {
            $order->add_order_note($data['notes']);
        }

        // Update order meta data
        if (!empty($data['order_meta'])) {
            foreach ($data['order_meta'] as $meta) {
                $order->update_meta_data($meta['key'], $meta['value']);
            }
        }

        // Convert back the VegaVend fields to their original values
        $order->update_meta_data('_vegavend_order_number', $data['vegavend_order_number']);
        if (isset($data['customer_id'])) {
            $order->update_meta_data('_vegavend_customer_id', $data['customer_id']);
        }

        foreach ($data['line_items'] as $item_data) {
            $item_id = $item_data['id'];
            $item = $order->get_item($item_id);
            if ($item) {
                $product_id = $item_data['vegavend_pid'];
                $parent_id = $item_data['vegavend_parent_pid'];
                $variation_id = $item_data['vegavend_variation_id'];

                $item->update_meta_data('_vegavend_pid', $product_id);
                $item->update_meta_data('_vegavend_parent_pid', $parent_id);
                $item->update_meta_data('_vegavend_variation_id', $variation_id);
                $item->save();
            }
        }

        $order->save();

        // Re-enable email notifications
        remove_filter('woocommerce_email_enabled_customer_processing_order', '__return_false');
        remove_filter('woocommerce_email_enabled_customer_completed_order', '__return_false');
        remove_filter('woocommerce_email_enabled_customer_invoice', '__return_false');
        remove_filter('woocommerce_email_enabled_customer_note', '__return_false');

        return rest_ensure_response(array(
            'status' => 'success',
            'message' => __('Order updated successfully', 'vegavend-merchant-connector'),
        ));
    }

    private function sanitize_recursive($data) {
        if (is_array($data)) {
            foreach ($data as $key => $value) {
                $data[$key] = $this->sanitize_recursive($value);
            }
        } else {
            $data = sanitize_text_field($data);
        }
        return $data;
    }

    public function on_order_update($order_id) {
        $order = wc_get_order($order_id);
        if ($order && $order->get_meta('_vegavend_order_number')) {
            // Disable email notifications for VegaVend orders
            add_filter('woocommerce_email_enabled', array($this, 'disable_emails_for_vegavend_orders'), 10, 2);
            $this->send_updated_order_to_vegavend($order);
        }
    }

    // Send updated order details back to VegaVend
    public function send_updated_order_to_vegavend($order) {
        $api_url = Vegavend_API_Config::get_order_url();
        $api_key = Vegavend_API_Config::get_api_key();

        $vegavend_order_number = $order->get_meta('_vegavend_order_number');
        if (empty($vegavend_order_number)) {
            return;
        }

        $order_data = array(
            'order_id' => $order->get_order_number(), // Use vegavend_order_number
            'vegavend_order_number' => $vegavend_order_number,
            'billing_address' => $order->get_address('billing'),
            'shipping_address' => $order->get_address('shipping'),
            'total_price' => $order->get_total(),
            'line_items' => array(),
            'customer_id' => $order->get_customer_id(), // Use vegavend_customer_id
            'status' => $order->get_status(),
            'notes' => $order->get_customer_note(),
            'order_date' => $order->get_date_created() ? $order->get_date_created()->date('Y-m-d H:i:s') : '',
            'modified_date' => $order->get_date_modified() ? $order->get_date_modified()->date('Y-m-d H:i:s') : '',
            'order_meta' => $order->get_meta_data(),
            'shipping_method' => $order->get_shipping_method(),
            'shipping_total' => $order->get_shipping_total(),
            'shipping_tax' => $order->get_shipping_tax(),
            'total_tax' => $order->get_total_tax(),
            'payment_method' => $order->get_payment_method(), // Include payment method
        );

        foreach ($order->get_items() as $item_id => $item) {
            $product = $item->get_product();
            $order_data['line_items'][] = array(
                'id' => $item->get_product_id(), // Use vegavend_pid
                'name' => $item->get_name(),
                'quantity' => $item->get_quantity(),
                'total' => $item->get_total(),
                'price' => $item->get_subtotal(),
                'sku' => $product ? $product->get_sku() : '',
                'gtin' => get_post_meta($product->get_id(), '_gtin', true),
                'mpn' => get_post_meta($product->get_id(), '_mpn', true),
                'original_product_id' => get_post_meta($product->get_id(), '_original_product_id', true),
                'product_type' => $product->get_type(),
                'parent_pid' => $product->get_parent_id(), // Use vegavend_parent_pid
                'variation_id' => $product->get_id(), // Use vegavend_variation_id
            );
        }

        $args = array(
            'method' => 'POST',
            'timeout' => 120,
            'redirection' => 5,
            'httpversion' => '1.0',
            'blocking' => true,
            'headers' => array(
                'Content-Type' => 'application/json',
                'X-API-Key' => $api_key,
            ),
            'body' => wp_json_encode($order_data),
        );

        $response = wp_remote_post($api_url, $args);

        $logger = wc_get_logger();
        $context = array('source' => 'vegavend-merchant-connector');

        if (is_wp_error($response)) {
            // Log error
            $logger->error('Error sending order to VegaVend: ' . $response->get_error_message(), $context);
        } else {
            $response_code = wp_remote_retrieve_response_code($response);
            $response_body = wp_remote_retrieve_body($response);

            if ($response_code !== 200) {
                // Log error
                $logger->error('Error response from VegaVend: ' . $response_body, $context);
            }
        }
    }

    public function disable_emails_for_vegavend_orders($enabled, $email) {
        $order = $email->object;
        if ($order instanceof WC_Order) {
            $vegavend_order_number = $order->get_meta('_vegavend_order_number');
            if (!empty($vegavend_order_number)) {
                return false;
            }
        }
        return $enabled;
    }
}

new Vegavend_Order_Update();