<?php
if (!defined('ABSPATH')) exit;

class Vegavend_Product_Sync {

    public function __construct() {
        add_action('init', array($this, 'schedule_cron_jobs'));
        add_action('vegavend_sync_single_product', array($this, 'sync_single_product'), 10, 2);
        add_action('wp_ajax_vegavend_sync_all_products', array($this, 'sync_all_products'));
    }

    public function schedule_cron_jobs() {
        if (!as_next_scheduled_action('vegavend_sync_products')) {
            as_schedule_recurring_action(time(), DAY_IN_SECONDS, 'vegavend_sync_products');
        }
    }

    // This function is used to sync products with the cron job
    public function vegavend_sync_products() {
        $sync_with_cron = get_option('vegavend_sync_with_cron');
        if ($sync_with_cron) {
            $vendor_status = $this->check_vendor_status();
            if ($vendor_status['valid_subscription']) {
                $total_products = $this->get_total_products();
                $allowed_products = $vendor_status['allowed_products'];

                // If allowed_products is "unlimited", sync all products
                if ($allowed_products === 'unlimited') {
                    $products_to_sync = $total_products;
                } else {
                    $products_to_sync = array_slice($total_products, 0, $allowed_products);
                }

                foreach ($products_to_sync as $index => $product_id) {
                    $product_status = get_post_status($product_id);
                    $timestamp = time() + ($index * 30);
                    $hook = 'vegavend_sync_single_product';
                    if (!as_next_scheduled_action($hook, array($product_id, $product_status), 'vegavend_group')) {
                        $scheduled = as_schedule_single_action($timestamp, $hook, array($product_id, $product_status), 'vegavend_group');
                    }
                }

                // Trigger action after all products have been scheduled
                do_action('vegavend_all_products_synced');
            }
        }
    }

    public function sync_single_product($product_id, $product_status = 'publish') {
        // Logic to sync a single product

        // Check if the product status is 'publish'
        if ($product_status !== 'publish') {
            return;
        }

        // Retrieve the product
        $product = wc_get_product($product_id);
        if ($product) {
            // Perform the synchronization logic

            // Ensure the Vegavend_Products class is instantiated
            if (class_exists('Vegavend_Products')) {
                $vegavend_products = new Vegavend_Products();
                $vegavend_products->sync_product_with_vegavend($product_id);
            }

            // Trigger custom hooks to update the product
            do_action('vegavend_update_product', $product_id);
            do_action('vegavend_save_post_product', $product_id, $product, true);
            do_action('vegavend_save_post', $product_id, $product, true);
            $product->save();
        }
    }

    public function sync_all_products() {
        if (!isset($_POST['vegavend_nonce']) || !check_ajax_referer('vegavend_nonce_action', 'vegavend_nonce', false)) {
            wp_send_json_error(array('message' => 'Invalid nonce'));
        }

        // Sanitize inputs
        $vegavend_nonce = sanitize_text_field(wp_unslash($_POST['vegavend_nonce']));

        // Check if a sync is already running
        if (get_transient('vegavend_sync_running')) {
            wp_send_json_error(array('message' => 'Sync already running'));
        }

        // Set transient to indicate sync is running
        set_transient('vegavend_sync_running', true, 60 * 60); // 1 hour expiration

        $total_products = $this->get_total_products();
        $current_time = time();

        // Clear and reschedule events if they have completed their run
        foreach ($total_products as $product) {
            $product_id = $product->ID;
            $next_scheduled = as_next_scheduled_action('vegavend_sync_single_product', array($product_id, 'additional_argument'));
            if ($next_scheduled && $next_scheduled <= $current_time) {
                as_unschedule_action('vegavend_sync_single_product', array($product_id, 'additional_argument'));
            }
        }

        // Check the number of scheduled events
        $scheduled_events = as_get_scheduled_actions(array('hook' => 'vegavend_sync_single_product'));
        if (is_array($scheduled_events)) {
        }

        // Schedule new events
        foreach ($total_products as $index => $product) {
            $product_id = $product->ID;
            $product_status = get_post_status($product_id);
            $timestamp = $current_time + ($index * 60);
            $hook = 'vegavend_sync_single_product';
            $scheduled = as_schedule_single_action($timestamp, $hook, array($product_id, $product_status), 'vegavend_group');
        }

        // Trigger action after all products have been scheduled
        do_action('vegavend_all_products_synced');

        // Delete transient to indicate sync is complete
        delete_transient('vegavend_sync_running');

        wp_send_json_success();
    }

    private function check_vendor_status() {
        $store_id = get_option('vegavend_store_id');
        $url = Vegavend_API_Config::get_subscription_report_url();
        $body = wp_json_encode(array('store_id' => $store_id));
        $args = array(
            'method' => 'POST',
            'body' => $body,
            'headers' => array(
                'X-API-Key' => Vegavend_API_Config::get_api_key(),
                'Content-Type' => 'application/json',
            ),
            'timeout' => 60,
            'redirection' => 10,
        );
        $response = wp_remote_post($url, $args);
        if (is_wp_error($response)) {
            return array(
                'valid_subscription' => false,
                'allowed_products' => 0
            );
        }
        $subscription_data = json_decode(wp_remote_retrieve_body($response));
        if ($subscription_data->status === 'success') {
            return array(
                'valid_subscription' => true,
                'allowed_products' => $subscription_data->data->no_of_product === 'unlimited' ? 'unlimited' : (int) $subscription_data->data->no_of_product
            );
        } else {
            return array(
                'valid_subscription' => false,
                'allowed_products' => 0
            );
        }
    }

    private function get_total_products() {
        $sync_setting = get_option('vegavend_sync_setting', 'sync_all');
        $meta_query = array();

        if ($sync_setting === 'sync_selected') {
            $meta_query = array(
                array(
                    'key' => '_vegavend_enable_sync',
                    'value' => 'on',
                ),
            );
        } else {
            $meta_query = array(
                'relation' => 'OR',
                array(
                    'key' => '_vegavend_enable_sync',
                    'value' => 'on',
                ),
                array(
                    'key' => '_vegavend_enable_sync',
                    'compare' => 'NOT EXISTS',
                ),
            );
        }

        $args = array(
            'post_type' => 'product',
            'posts_per_page' => -1,
            'meta_query' => $meta_query,
        );

        $query = new WP_Query($args);
        return $query->posts;
    }
}

new Vegavend_Product_Sync();
?>