<?php
if ( ! defined( 'ABSPATH' ) ) exit;

require_once plugin_dir_path(__FILE__) . 'vegavend_api_config.php';

class Vegavend_Products {

    public function __construct() {
        add_action('vegavend_save_post', array($this, 'on_product_save'), 10, 3);
        add_action('add_meta_boxes', array($this, 'add_vegavend_channel_meta_box'));
        add_action('vegavend_save_post_product', array($this, 'save_vegavend_channel_meta_box'));
        add_action('rest_api_init', array($this, 'register_product_api_route'));
        add_action('trashed_post', array($this, 'on_product_trashed'), 10, 1); // Add action for trashed post
        add_action('vegavend_save_post_product', array($this, 'on_product_save'), 10, 3);
        
        // Add WooCommerce hooks to trigger custom hooks
        add_action('save_post_product', array($this, 'trigger_custom_hooks'), 10, 3);
        add_action('save_post', array($this, 'trigger_custom_hooks'), 10, 3);
    }

    // Trigger custom hooks
    public function trigger_custom_hooks($post_id, $post, $update) {
        if ('product' === $post->post_type) {
            do_action('vegavend_save_post', $post_id, $post, $update);
            do_action('vegavend_save_post_product', $post_id, $post, $update);
        }
    }

    // Create Product on Product save
    public function on_product_save($post_id, $post, $update) {
        // Check if this is an autosave
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
    
        // Check if this is a revision
        if (wp_is_post_revision($post_id)) {
            return;
        }
    
        // Check the post type
        if ('product' !== $post->post_type) {
            return;
        }
    
        // Verify the nonce
        if (!isset($_POST['vegavend_product_nonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['vegavend_product_nonce'])), 'vegavend_save_product_data')) {
            return;
        }
    
        // Check user permissions
        if (!current_user_can('edit_post', $post_id)) {
            return;
        }
    
        // Sync product with VegaVend if enabled
        if (get_post_meta($post_id, '_vegavend_enable_sync', true) === 'on') {
            $this->sync_product_with_vegavend($post_id);
        }
    }

    // Handle product trashed
    public function on_product_trashed($post_id) {
        // Check the post type
        if ('product' !== get_post_type($post_id)) {
            return;
        }

        // Remove product from VegaVend
        $this->remove_product_from_vegavend($post_id);
    }

    // Sync Product with VegaVend
    public function sync_product_with_vegavend($post_id) {
        $store_id = get_option('vegavend_store_id');
        if (!$store_id) {
            return;
        }

        $product = wc_get_product($post_id);
        $product_data = $this->prepare_product_data($product);

        $response = $this->send_product_to_vegavend($product_data);

        if (!is_wp_error($response)) {
            update_post_meta($post_id, '_vegavend_last_sync', current_time('mysql'));
        }
    }

    // Remove Product from VegaVend
    private function remove_product_from_vegavend($post_id) {
        $store_id = get_option('vegavend_store_id');
        if (!$store_id) {
            return;
        }

        $product = wc_get_product($post_id);
        $product_data = array(
            'store_id' => $store_id,
            'id' => $product->get_id(),
        );

        $response = $this->send_product_removal_to_vegavend($product_data);

        if (!is_wp_error($response)) {
            delete_post_meta($post_id, '_vegavend_last_sync');
            wp_delete_post($post_id, true); // Delete the product from WooCommerce
        }
    }

    // Send Product Removal to VegaVend
    private function send_product_removal_to_vegavend($product_data) {
        $api_key = Vegavend_API_Config::get_api_key();
        $api_url = Vegavend_API_Config::get_product_url();
        $store_id = get_option('vegavend_store_id'); // Ensure store_id is included

        $product_data['store_id'] = $store_id; // Ensure store_id is included in product data

        $args = array(
            'method' => 'DELETE',
            'timeout' => 60, // Increase timeout to 60 seconds
            'redirection' => 5,
            'httpversion' => '1.0',
            'blocking' => true,
            'headers' => array(
                'Content-Type' => 'application/json',
                'X-API-Key' => $api_key,
            ),
            'body' => wp_json_encode($product_data),
        );

        $response = wp_remote_request($api_url, $args);

        if (is_wp_error($response)) {
            return new WP_Error('vegavend_api_error', $response->get_error_message());
        }

        $response_code = wp_remote_retrieve_response_code($response);
        $response_body = wp_remote_retrieve_body($response);

        if ($response_code !== 200) {
            return new WP_Error('vegavend_api_error', sprintf(
                /* translators: %d: Error code. */
                __('VegaVend API returned error code: %d', 'vegavend-merchant-connector'),
                $response_code
            ));
        }

        $result = json_decode($response_body, true);

        if (!$result || !isset($result['status']) || $result['status'] !== 'success') {
            return new WP_Error('vegavend_api_error', __('Invalid response from VegaVend API', 'vegavend-merchant-connector'));
        }

        return $result;
    }

    // Prepare Product Data
    private function prepare_product_data($product) {
        $store_id = get_option('vegavend_store_id');
        $date_sale_from = $product->get_date_on_sale_from();
        $date_sale_to = $product->get_date_on_sale_to();
        $global_manage_stock = get_option('woocommerce_manage_stock') === 'yes'; // Include global stock management setting
        $tax_status = get_option('woocommerce_calc_taxes') === 'yes' ? $product->get_tax_status() : 'none'; // Check global tax setting

        $product_data = array(
            'store_id' => $store_id,
            'id' => $product->get_id(),
            'type' => $product->get_type(),
            'name' => $product->get_name(),
            'product_status' => $product->get_status(),
            'regular_price' => $product->get_regular_price(),
            'sale_price' => $product->get_sale_price(),
            'date_on_sale_from' => (isset($date_sale_from) && !empty($date_sale_from)) ? $date_sale_from->format('Y-m-d H:i:s') : $date_sale_from,
            'date_on_sale_to' => (isset($date_sale_to) && !empty($date_sale_to)) ? $date_sale_to->format('Y-m-d H:i:s') : $date_sale_to,
            'description' => $product->get_description(),
            'short_description' => $product->get_short_description(),
            'sku' => $product->get_sku(),
            'stock_quantity' => $product->get_stock_quantity(),
            'manage_stock' => $product->get_manage_stock(),
            'stock_status' => $product->get_stock_status(),
            'backorders' => $product->get_backorders(),
            'catalog_visibility' => $product->get_catalog_visibility(),
            'weight' => $product->get_weight(),
            'length' => $product->get_length(),
            'width' => $product->get_width(),
            'height' => $product->get_height(),
            'shipping_class' => $product->get_shipping_class(),
            'downloadable' => $product->is_downloadable(),
            'virtual' => $product->is_virtual(),
            'featured' => $product->is_featured(),
            'on_sale' => $product->is_on_sale(),
            'average_rating' => $product->get_average_rating(),
            'rating_count' => $product->get_rating_count(),
            'total_sales' => $product->get_total_sales(),
            'related_ids' => wc_get_related_products($product->get_id()),
            'categories' => $this->get_product_categories($product),
            'tags' => $this->get_product_tags($product),
            'upsell_ids' => $product->get_upsell_ids(),
            'cross_sell_ids' => $product->get_cross_sell_ids(),
            'parent_id' => $product->get_parent_id(),
            'purchase_note' => $product->get_purchase_note(),
            'menu_order' => $product->get_menu_order(),
            'reviews_allowed' => $product->get_reviews_allowed(),
            'attributes' => $this->get_product_attributes($product),
            'images' => $this->get_product_images($product),
            'thumbnail' => wp_get_attachment_image_url($product->get_image_id(), 'full'),
            'global_manage_stock' => $global_manage_stock, // Include global stock management setting

            // Add gallery images
            'gallery_images' => array_map(function($image_id) {
                return wp_get_attachment_image_url($image_id, 'full');
            }, $product->get_gallery_image_ids()),
            'thumbnail_id' => $product->get_image_id(),
            'product_addons' => array_filter((array) $product->get_meta('_product_addons')),
            'exclude_global' => $product->get_meta('_product_addons_exclude_global'),
            'sync_status' => get_post_meta($product->get_id(), '_vegavend_sync_status', true), // Add sync_status
            'tax_class' => $product->get_tax_class(), // Add tax_class
            'tax_status' => $tax_status, // Add tax_status
        );

        // Include GTIN and MPN for simple products only
        if (!$product->is_type('variable')) {
            $product_data['gtin'] = get_post_meta($product->get_id(), '_gtin', true);
            $product_data['mpn'] = get_post_meta($product->get_id(), '_mpn', true);
        }

        // Include additional fields for variable products
        if ($product->is_type('variable')) {
            $product_data['variations'] = $this->get_product_variations($product);
        }

        return $product_data;
    }

    // Get product categories
    private function get_product_categories($product) {
        $categories = array();
        $terms = get_the_terms($product->get_id(), 'product_cat');
        if ($terms && !is_wp_error($terms)) {
            foreach ($terms as $term) {
                $term_meta = get_term_meta($term->term_id, 'vegavend_product_category', true);
                if($term_meta && !empty($term_meta)){
                    $categories[] = $term_meta;
                }
            }
        }
        return $categories;
    }

    // Get product tags
    private function get_product_tags($product) {
        $tags = array();
        $terms = get_the_terms($product->get_id(), 'product_tag');
        if ($terms && !is_wp_error($terms)) {
            foreach ($terms as $term) {
                $tags[] = $term->name;
            }
        }
        return $tags;
    }

    // Get Product Images
    private function get_product_images($product) {
        $images = array();
        $attachment_ids = $product->get_gallery_image_ids();
        
        if ($product->get_image_id()) {
            $images[] = wp_get_attachment_url($product->get_image_id());
        }

        foreach ($attachment_ids as $attachment_id) {
            $images[] = wp_get_attachment_url($attachment_id);
        }

        return $images;
    }

    // Get Product Attributes
    private function get_product_attributes($product) {
        $attributes = array();
        
        foreach ($product->get_attributes() as $attribute) {
            $attribute_data = array(
                'name' => $this->vegavend_human_readable_attribute_name($attribute->get_name()), // Use human-readable attribute name
                'options' => array_map(function($option) use ($attribute) {
                    if ($attribute->is_taxonomy()) {
                        $term = get_term_by('id', $option, $attribute->get_name());
                        return $term ? $term->name : $option; // Ensure attribute value is set as name
                    } else {
                        return $option; // Return the option value if it's not a taxonomy
                    }
                }, $attribute->get_options()),
                'position' => $attribute->get_position(),
                'visible' => $attribute->get_visible(),
                'variation' => $attribute->get_variation(), // Include 'Used for variations' information
            );
            $attributes[] = $attribute_data;
        }

        // Add GTIN to attributes if available
        $gtin = get_post_meta($product->get_id(), '_gtin', true);
        if (!empty($gtin)) {
            $attributes[] = array(
                'name' => 'gtin',
                'options' => [$gtin],
                'position' => 0,
                'visible' => true,
                'variation' => false,
            );
        }

        // Add MPN to attributes if available
        $mpn = get_post_meta($product->get_id(), '_mpn', true);
        if (!empty($mpn)) {
            $attributes[] = array(
                'name' => 'mpn',
                'options' => [$mpn],
                'position' => 0,
                'visible' => true,
                'variation' => false,
            );
        }

        return $attributes;
    }

    private function get_product_variations($product) {
        $variations = array();
        if ($product->is_type('variable')) {
            foreach ($product->get_available_variations() as $variation) {
                $variation_id = $variation['variation_id'];
                $variation_obj = new WC_Product_Variation($variation_id);
                $global_manage_stock = get_option('woocommerce_manage_stock') === 'yes'; // Include global stock management setting

                $variation_data = array(
                    'id' => $variation_id,
                    'variation_id' => $variation_id,
                    'attributes' => array_map(function($attribute_key, $attribute_value) {
                        // Remove 'attribute_' and 'pa_' prefixes
                        $taxonomy = preg_replace('/^attribute_|^pa_/', '', $attribute_key);
                        $human_readable_name = $this->vegavend_human_readable_attribute_name($taxonomy);
                        if (taxonomy_exists($taxonomy)) {
                            $term = get_term_by('name', $attribute_value, $taxonomy); // Use the name instead of the slug
                            $attribute_value = $term ? $term->name : $attribute_value;
                        }
                        return array($human_readable_name => $attribute_value);
                    }, array_keys($variation['attributes']), $variation['attributes']),
                    'price' => $variation_obj->get_price(),
                    'regular_price' => $variation_obj->get_regular_price(),
                    'sale_price' => $variation_obj->get_sale_price(),
                    'sku' => $variation_obj->get_sku(),
                    'stock_quantity' => $variation_obj->get_stock_quantity(),
                    'gtin' => get_post_meta($variation_id, '_gtin', true),
                    'mpn' => get_post_meta($variation_id, '_mpn', true),
                    'weight' => $variation_obj->get_weight(),
                    'length' => $variation_obj->get_length(),
                    'width' => $variation_obj->get_width(),
                    'height' => $variation_obj->get_height(),
                    'image' => wp_get_attachment_url($variation_obj->get_image_id()),
                    'manage_stock' => $variation_obj->get_manage_stock(),
                    'stock_status' => $variation_obj->get_stock_status(),
                    'backorders' => $variation_obj->get_backorders(),
                    'description' => $variation_obj->get_description(),
                    'global_manage_stock' => $global_manage_stock, // Include global stock management setting
                );
                $variations[] = $variation_data;
            }
        }
        return $variations;
    }

    private function send_product_to_vegavend($product_data) {
        $api_key = Vegavend_API_Config::get_api_key();
        $api_url = Vegavend_API_Config::get_product_url();
        $store_id = get_option('vegavend_store_id'); // Ensure store_id is included

        $product_data['store_id'] = $store_id; // Ensure store_id is included in product data

        $args = array(
            'method' => 'POST',
            'timeout' => 60, // Increase timeout to 60 seconds
            'redirection' => 5,
            'httpversion' => '1.0',
            'blocking' => true,
            'headers' => array(
                'Content-Type' => 'application/json',
                'X-API-Key' => $api_key,
            ),
            'body' => wp_json_encode($product_data),
        );

        $response = wp_remote_post($api_url, $args);

        if (is_wp_error($response)) {
            return new WP_Error('vegavend_api_error', $response->get_error_message());
        }

        $response_code = wp_remote_retrieve_response_code($response);
        $response_body = wp_remote_retrieve_body($response);

        if ($response_code !== 200) {
            return new WP_Error('vegavend_api_error', sprintf(
                /* translators: %d: Error code. */
                __('VegaVend API returned error code: %d', 'vegavend-merchant-connector'),
                $response_code
            ));
        }

        $result = json_decode($response_body, true);

        if (!$result || !isset($result['status']) || $result['status'] !== 'success') {
            return new WP_Error('vegavend_api_error', __('Invalid response from VegaVend API', 'vegavend-merchant-connector'));
        }

        return $result;
    }

    // Add VegaVend Channel Meta Box
    public function add_vegavend_channel_meta_box() {
        add_meta_box(
            'vegavend_channel_meta_box',
            esc_html__('VegaVend Channel', 'vegavend-merchant-connector'),
            array($this, 'render_vegavend_channel_meta_box'),
            'product',
            'side',
            'default'
        );
    }

    // Render VegaVend Channel Meta Box
    public function render_vegavend_channel_meta_box($post) {
        wp_nonce_field('vegavend_meta_box_nonce_action', 'vegavend_meta_box_nonce');
        $enable_sync = get_post_meta($post->ID, '_vegavend_enable_sync', true);
        if ($enable_sync == '') {
            $enable_sync = 'on';
        }
        ?>
        <label for="vegavend_enable_sync"><?php esc_html_e('Enable product to sync with VegaVend:', 'vegavend-merchant-connector'); ?></label>
        <select id="vegavend_enable_sync" name="vegavend_enable_sync" style="width: 100%; margin-top: 10px;">
            <option value="on" <?php selected($enable_sync, 'on'); ?>><?php esc_html_e('Allow Sync', 'vegavend-merchant-connector'); ?></option>
            <option value="off" <?php selected($enable_sync, 'off'); ?>><?php esc_html_e('Don\'t Sync', 'vegavend-merchant-connector'); ?></option>
        </select>
        <?php
        // Add nonce field for product save
        wp_nonce_field('vegavend_save_product_data', 'vegavend_product_nonce');
    }

    // Save VegaVend Channel Meta Box
    public function save_vegavend_channel_meta_box($post_id) {
        if (!isset($_POST['vegavend_meta_box_nonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['vegavend_meta_box_nonce'])), 'vegavend_meta_box_nonce_action')) {
            return;
        }

        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }

        if (!current_user_can('edit_post', $post_id)) {
            return;
        }

        $enable_sync = isset($_POST['vegavend_enable_sync']) ? sanitize_text_field(wp_unslash($_POST['vegavend_enable_sync'])) : 'off';
        update_post_meta($post_id, '_vegavend_enable_sync', $enable_sync);

        // Remove product from VegaVend if sync is disabled
        if ($enable_sync === 'off') {
            $this->remove_product_from_vegavend($post_id);
        }
    }

    // Register REST API routes
    public function register_product_api_route() {
        register_rest_route(Vegavend_API_Config::BASE_ROUTE, '/products/', array(
            'methods'  => 'POST',
            'callback' => array($this, 'handle_vegavend_api_request'),
            'permission_callback' => array($this, 'check_api_key_permission'),
        ));

        register_rest_route(Vegavend_API_Config::BASE_ROUTE, '/get-categories/', array(
            'methods' => 'POST',
            'callback' => array($this, 'get_categories'),
            'permission_callback' => array($this, 'check_api_key_permission'),
        ));
    }

    // Check API key permission
    public function check_api_key_permission($request) {
        $provided_api_key = $request->get_header('X-API-Key');
        if ($provided_api_key !== Vegavend_API_Config::get_api_key()) {
            return new WP_Error('invalid_api_key', __('Invalid API key.', 'vegavend-merchant-connector'), array('status' => 401));
        }
        return true;
    }

    // Handle WooCommerce API request
    public function handle_vegavend_api_request($request) {
        $data = $request->get_json_params();

        if (isset($data['batch_size']) && isset($data['offset'])) {
            return $this->handle_vegavend_bulk_request($request);
        }
        return $this->handle_single_product_request($data);
    }

    // Handle single product request
    private function handle_single_product_request($data) {
        if (!isset($data['product_id'])) {
            return new WP_Error('missing_product_id', __('Missing product ID', 'vegavend-merchant-connector'), array('status' => 400));
        }

        $product_id = intval($data['product_id']);
        $product = wc_get_product($product_id);

        if (!$product) {
            return new WP_Error('invalid_product_id', __('Invalid product ID', 'vegavend-merchant-connector'), array('status' => 404));
        }

        $product_data = $this->prepare_product_data($product);
        $response = $this->send_product_to_vegavend($product_data);

        if (is_wp_error($response)) {
            return new WP_Error('vegavend_sync_failed', $response->get_error_message(), array('status' => 500));
        }

        return rest_ensure_response(array(
            'status' => 'success',
            'message' => __('Product synced successfully', 'vegavend-merchant-connector'),
            'product' => $product_data,
            'response' => $response
        ));
    }

    // Handle bulk import request
    public function handle_vegavend_bulk_request($request) {
        $batch_size = isset($request['batch_size']) ? intval($request['batch_size']) : 5; // Change default batch size to 5
        $offset = isset($request['offset']) ? intval($request['offset']) : 0; // Initialize offset to 0
        $store_id = get_option('vegavend_store_id'); // Ensure store_id is included

        $args = array(
            'post_type' => 'product',
            'posts_per_page' => $batch_size,
            'offset' => $offset,
            'meta_query' => array(
                array(
                    'relation' => 'OR',
                    array(
                        'key' => '_vegavend_enable_sync',
                        'value' => 'on',
                    ),
                    array(
                        'key' => '_vegavend_enable_sync',
                        'compare' => 'NOT EXISTS',
                    )
                )
            )
        );

        $products = new WP_Query($args);
        $response = array();

        if ($products->have_posts()) {
            while ($products->have_posts()) {
                $products->the_post();
                $product = wc_get_product(get_the_ID());

                $product_data = $this->prepare_product_data($product);
                $product_data['store_id'] = $store_id; // Ensure store_id is included in product data

                $response['products'][] = $product_data;
            }
            wp_reset_postdata();

            $total_products = $products->found_posts;
            $next_products = ($offset + $batch_size < $total_products) ? 'yes' : 'no';
            $response['nextProducts'] = $next_products;
            $response['status'] = 'success';
        } else {
            $response['message'] = esc_html__('No products found', 'vegavend-merchant-connector');
            $response['nextProducts'] = 'no';
            $response['status'] = 'error';
        }

        return rest_ensure_response($response);
    }

    // Handle get categories request
    public function get_categories() {
        $categories = get_terms(array(
            'taxonomy' => 'product_cat',
            'hide_empty' => false,
        ));

        if (is_wp_error($categories)) {
            return new WP_Error('categories_fetch_failed', __('Failed to fetch categories', 'vegavend-merchant-connector'), array('status' => 500));
        }

        $categories_data = array();
        foreach ($categories as $category) {
            $categories_data[] = array(
                'id' => $category->term_id,
                'name' => $category->name,
            );
        }

        return rest_ensure_response(array(
            'status' => 'success',
            'categories' => $categories_data,
        ));
    }

    // Function to get the human-readable attribute name
    private function vegavend_human_readable_attribute_name($taxonomy) {
        global $wpdb;
        $cache_key = 'human_readable_attribute_name_' . $taxonomy;
        $attribute_name = wp_cache_get($cache_key);

        if ($attribute_name === false) {
            $attribute_name = $wpdb->get_var($wpdb->prepare("
                SELECT attribute_label 
                FROM {$wpdb->prefix}woocommerce_attribute_taxonomies 
                WHERE attribute_name = %s
            ", str_replace('pa_', '', $taxonomy)));

            wp_cache_set($cache_key, $attribute_name);
        }

        return $attribute_name ? $attribute_name : $taxonomy;
    }
}

new Vegavend_Products();