<?php
/*
 Plugin Name: VegaVend Merchant Connector
 Plugin URI: https://vegavend.com/vendor-help/integration-connectors/woocommece-connector/
 Description: A plugin that seamlessly integrates and synchronises your products into the VegaVend marketplace.
 Version: 1.0.4
 Author: VegaVend - The Online Marketplace
 Author URI: https://vegavend.com/about-us/
 Requires at least: 5.0
 Tested up to: 6.7
 Requires PHP: 7.4
 Stable tag: 1.0.0
 License: GPLv2 or later
 License URI: https://www.gnu.org/licenses/gpl-2.0.html
 Text Domain: vegavend-merchant-connector
 Domain Path: /languages
*/

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

require_once plugin_dir_path(__FILE__) . 'includes/vegavend_api_config.php';
require_once plugin_dir_path(__FILE__) . 'includes/vegavend_admin_menu.php';
require_once plugin_dir_path(__FILE__) . '/includes/vegavend-ajax-handlers.php';

class Vegavend_Merchant_Connector {
    private $version = '1.0.4';
    private $api_key;
    private $api_endpoint;

    public function __construct() {
        $this->version = '1.0.0';
        $this->api_key = Vegavend_API_Config::get_api_key();
        $this->api_endpoint = Vegavend_API_Config::get_product_url();
        $this->define_constants();
        $this->include_files();
        $this->init_hooks();
    }

    private function define_constants() {
        define('VEGAVEND_VERSION', $this->version);
        define('VEGAVEND_PLUGIN_URL', untrailingslashit(plugins_url('', __FILE__)));
        define('VEGAVEND_PLUGIN_DIR', untrailingslashit(plugin_dir_path(__FILE__)));
        define('VEGAVEND_IMAGES', VEGAVEND_PLUGIN_URL . '/assets/images');
    }

    private function include_files() {
        require_once VEGAVEND_PLUGIN_DIR . '/admin/categories.php';
        require_once VEGAVEND_PLUGIN_DIR . '/admin/woocommerce.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_order_import.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_order_update.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_products.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_product_sync.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_settings_page.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_status_page.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_support_page.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_product_issues_page.php';
        require_once VEGAVEND_PLUGIN_DIR . '/includes/vegavend_account_page.php';
    }

    private function init_hooks() {
        add_action('plugins_loaded', array($this, 'check_woocommerce_dependency'));
        add_action('admin_init', array($this, 'check_woocommerce_dependency'));
        
        if ($this->is_woocommerce_active()) {
            add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_assets'));
            add_action('admin_init', array($this, 'register_settings'));
            add_action('rest_api_init', array($this, 'handle_preflight'), 15);
            add_filter('rest_pre_serve_request', array($this, 'add_cors_headers'), 15);
        }

        // Check if store ID is set
        if (!get_option('vegavend_store_id') && !$this->is_notice_dismissed()) {
            add_action('admin_notices', array($this, 'store_not_connected_notice'));
        }

        // Handle dismiss notice action
        add_action('wp_ajax_dismiss_vegavend_notice', array($this, 'dismiss_vegavend_notice'));
    }

    private function is_notice_dismissed() {
        $dismissed_time = get_option('vegavend_dismiss_notice_time');
        if ($dismissed_time) {
            $current_time = current_time('timestamp');
            $time_diff = $current_time - $dismissed_time;
            return $time_diff < 7 * DAY_IN_SECONDS;
        }
        return false;
    }

    public function check_woocommerce_dependency() {
        if (!$this->is_woocommerce_active()) {
            add_action('admin_notices', array($this, 'woocommerce_missing_notice'));
        }
    }

    private function is_woocommerce_active() {
        return in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')));
    }

    public function woocommerce_missing_notice() {
        ?>
        <div class="notice notice-error">
            <p><?php esc_html_e('VegaVend Merchant Connector plugin requires WooCommerce to be active.', 'vegavend-merchant-connector'); ?></p>
        </div>
        <?php
    }

    public function enqueue_admin_assets($hook) {
        if ('toplevel_page_vegavend-connector' !== $hook) {
            return;
        }

        wp_localize_script('vegavend-admin-script', 'vegavendData', array(
            'nonce' => wp_create_nonce('vegavend-admin-nonce')
        ));
    }

    public function register_settings() {
        register_setting('vegavend_options_group', 'vegavend_api_key', array(
            'type' => 'string',
            'sanitize_callback' => array($this, 'sanitize_api_key'),
        ));
    }

    public function sanitize_api_key($input) {
        return sanitize_text_field($input);
    }

    public function handle_preflight() {
        if (isset($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
            if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_METHOD']) && in_array($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_METHOD'], array('GET', 'POST', 'PUT', 'DELETE'))) {
                header('Access-Control-Allow-Origin: *');
                header('Access-Control-Allow-Methods: POST, GET, OPTIONS, PUT, DELETE');
                header('Access-Control-Allow-Credentials: true');
                header('Access-Control-Allow-Headers: Authorization, Content-Type, X-Api-Key, X-Store-Id');
                exit;
            }
        }
    }

    public function add_cors_headers($value) {
        header('Access-Control-Allow-Origin: *');
        header('Access-Control-Allow-Methods: POST, GET, OPTIONS, PUT, DELETE');
        header('Access-Control-Allow-Credentials: true');
        header('Access-Control-Allow-Headers: Authorization, Content-Type, X-Api-Key, X-Store-Id');
        return $value;
    }

    public function store_not_connected_notice() {
        ?>
        <div class="notice notice-warning is-dismissible">
            <p><?php esc_html_e('VegaVend Merchant Connector plugin requires a valid Store ID to be connected.', 'vegavend-merchant-connector'); ?></p>
            <p>
                <a href="<?php echo esc_url(admin_url('admin.php?page=vegavend-settings')); ?>" class="button button-primary"><?php esc_html_e('Connect', 'vegavend-merchant-connector'); ?></a>
                <button type="button" class="button" id="dismiss-vegavend-notice"><?php esc_html_e('Dismiss for 7 days', 'vegavend-merchant-connector'); ?></button>
            </p>
        </div>
        <?php

        $script = "
            document.getElementById('dismiss-vegavend-notice').addEventListener('click', function() {
                var data = {
                    action: 'dismiss_vegavend_notice',
                    nonce: '" . esc_js(wp_create_nonce('dismiss_vegavend_notice')) . "'
                };

                jQuery.post(ajaxurl, data, function(response) {
                    if (response.success) {
                        document.querySelector('.notice-warning').style.display = 'none';
                    }
                });
            });
        ";
        wp_add_inline_script('vegavend-admin-script', $script);
    }

    public function dismiss_vegavend_notice() {
        check_ajax_referer('dismiss_vegavend_notice', 'nonce');
        update_option('vegavend_dismiss_notice_time', current_time('timestamp'));
        wp_send_json_success();
    }
}

new Vegavend_Merchant_Connector();